@@ -3,14 +3,9 @@
 
 """Shows all times of day for the given time zones.
 
-This can be useful to select a common meeting time across multiple
-time zones easily. This takes into account daylight savings and
-whatnot, and can schedule meetings in the future. Default settings are
-taken from ~/.config/undertime.yml. Dates are parsed with the
-dateparser or parsedatetime modules, if available, in that order, see
-https://dateparser.readthedocs.io/en/latest/ and
-https://github.com/bear/parsedatetime.
-"""
+This helps to find a common meeting time across multiple time
+zones. It defaults to "now" but can look at other dates with the
+"WHEN" argument."""
 
 # Copyright (C) 2017 Antoine Beaupré
 #
@@ -29,40 +24,25 @@ https://github.com/bear/parsedatetime.
 
 import argparse
 import datetime
+import importlib
 import logging
 import logging.handlers
 import os
+import platform
 import re
 import sys
+from site import USER_BASE
 
 # also considered colorama and crayons
 # 1. colorama requires to send reset codes. annoying.
 # 2. crayons is a wrapper around colorama, not in debian
 import termcolor
 
-# we use dateparser to handle -d but also fallback on parsedatetime or
-# "now" if both are missing
-#
-# another option could be https://pypi.org/project/arrow/ which does
-# date parsing, `relativedate`, `pytz`, and more...
-try:
-    import dateparser
-except ImportError:  # pragma: no cover
-    dateparser = None
-    try:
-        import parsedatetime
-    except ImportError:
-        parsedatetime = None
-
-from dateutil.relativedelta import relativedelta
-
 # XXX: we *also* need pytz even though dateutil also ships time zone
 # info. pytz has the *list* of all time zones, which dateutil doesn't
 # ship, or at least not yet. This might eventually all get fixed in
-# the standard library, see:
-# https://lwn.net/SubscriberLink/813691/8d2dde9efd443c3b/
+# the standard library, see: https://lwn.net/Articles/813691/
 import pytz
-import yaml
 
 # for tabulated data, i looked at other alternatives
 # humanfriendly has a tabulator: https://humanfriendly.readthedocs.io/en/latest/#module-humanfriendly.tables
@@ -71,28 +51,34 @@ import yaml
 # terminaltables is the full thing: https://robpol86.github.io/terminaltables/
 # "rich" has more features, but awkward interface: https://rich.readthedocs.io/en/stable/reference/table.html
 # ie. you need to add rows one at a time, not worth it
-
+#
 # originally, i was just centering thing with the .format()
 # handler. this was working okay except that it was too wide because i
 # was using the widest column as width everywhere because i'm lazy.
-
+#
 # i switched to tabulate because terminaltables has problems with
 # colors, see https://gitlab.com/anarcat/undertime/issues/9 and
 # https://github.com/Robpol86/terminaltables/issues/55
 import tabulate
+import yaml
+from dateutil.relativedelta import relativedelta
 
 
 class ImportlibVersionAction(argparse._VersionAction):
     """Version action with a default from importlib"""
 
-    def __call__(self, *args, **kwargs):
+    @staticmethod
+    def _version():
         # call importlib only if needed, it takes 20ms to load
         try:
             import importlib.metadata as importlib_metadata
         except ImportError:
-            import importlib_metadata
+            import importlib_metadata  # type:ignore
 
-        self.version = importlib_metadata.version("undertime")
+        return importlib_metadata.version("undertime")
+
+    def __call__(self, *args, **kwargs):
+        self.version = self._version()
         return super().__call__(*args, **kwargs)
 
 
@@ -142,7 +128,16 @@ class ConfigAction(argparse._StoreAction):
         """change defaults for the namespace, still allows overriding
         from commandline options"""
         for path in values:
-            parser.set_defaults(**self.parse_config(path))
+            try:
+                # XXX: this is probably the bit that fails with
+                # subparsers and groups
+                parser.set_defaults(**self.parse_config(path))
+            except FileNotFoundError as e:
+                logging.debug("config file %s not found: %s", path, e)
+            else:
+                # stop processing once we find a valid configuration
+                # file
+                break
         super().__call__(parser, ns, values, option)
 
     def parse_config(self, path):
@@ -154,12 +149,17 @@ class YamlConfigAction(ConfigAction):
     """YAML config file parser action"""
 
     def parse_config(self, path):
+        """This doesn't handle errors around open() and others, callers should
+        probably catch FileNotFoundError at least.
+        """
         try:
             with open(os.path.expanduser(path), "r") as handle:
                 logging.debug("parsing path %s as YAML" % path)
                 return yaml.safe_load(handle) or {}
-        except (FileNotFoundError, yaml.parser.ParserError) as e:
-            raise argparse.ArgumentError(self, e)
+        except yaml.error.YAMLError as e:
+            raise argparse.ArgumentError(
+                self, "failed to parse YAML configuration: %s" % e
+            )
 
 
 class ConfigArgumentParser(argparse.ArgumentParser):
@@ -174,6 +174,9 @@ class ConfigArgumentParser(argparse.ArgumentParser):
 
             self.add_argument('--config', action=YamlConfigAction, default=self.default_config())
 
+    This shows how the configuration file overrides the default value
+    for an option:
+
     >>> from tempfile import NamedTemporaryFile
     >>> c = NamedTemporaryFile()
     >>> c.write(b"foo: delayed\\n")
@@ -187,10 +190,12 @@ class ConfigArgumentParser(argparse.ArgumentParser):
     True
     >>> args.foo
     'delayed'
+    >>> args = parser.parse_args(['--foo', 'quux'])
+    >>> args.foo
+    'quux'
 
     This is the same test, but with `--config` called earlier, which
-    should still work (but doesn't, because the subsequent action
-    changes the default).
+    should still work:
 
     >>> from tempfile import NamedTemporaryFile
     >>> c = NamedTemporaryFile()
@@ -208,6 +213,9 @@ class ConfigArgumentParser(argparse.ArgumentParser):
     >>> args = parser.parse_args(['--foo', 'baz'])
     >>> args.foo
     'baz'
+
+    This tests that you can override the config file defaults altogether:
+
     >>> parser = ConfigArgumentParser()
     >>> a = parser.add_argument('--config', action=YamlConfigAction, default=[c.name])
     >>> a = parser.add_argument('--foo', default='bar')
@@ -217,8 +225,22 @@ class ConfigArgumentParser(argparse.ArgumentParser):
     >>> args = parser.parse_args(['--config', '/dev/null', '--foo', 'baz'])
     >>> args.foo
     'baz'
-    >>> c.close()
 
+    This tests multiple search paths, first one should be loaded:
+
+    >>> from tempfile import NamedTemporaryFile
+    >>> d = NamedTemporaryFile()
+    >>> d.write(b"foo: argh\\n")
+    10
+    >>> d.flush()
+    >>> parser = ConfigArgumentParser()
+    >>> a = parser.add_argument('--config', action=YamlConfigAction, default=[d.name, c.name])
+    >>> a = parser.add_argument('--foo', default='bar')
+    >>> args = parser.parse_args([])
+    >>> args.foo
+    'argh'
+    >>> c.close()
+    >>> d.close()
     """
 
     def __init__(self, *args, **kwargs):
@@ -264,24 +286,26 @@ class ConfigArgumentParser(argparse.ArgumentParser):
                 logging.debug("not loading delayed action because of config override")
                 # action is already loaded, no need to parse it again
                 continue
-            try:
-                action(self, ns, action.default, None)
-                logging.debug("loaded config file: %s" % action.default)
-            except argparse.ArgumentError as e:
-                # ignore errors from missing default
-                logging.debug("default config file %s error: %s" % (action.default, e))
-        # this will actually load the relevant config file when found
+            logging.debug("searching config files: %s" % action.default)
+            action(self, ns, action.default, None)
         # on the commandline
         #
         # note that this will load the config file a second time
         return super().parse_args(args, namespace)
 
     def default_config(self):
-        """handy shortcut to detect commonly used config paths"""
+        """handy shortcut to detect commonly used config paths
+
+        This list is processed as a FIFO: if a file is found in there,
+        it will be parsed and the remaining ones will be ignored.
+        """
         return [
             os.path.join(
                 os.environ.get("XDG_CONFIG_HOME", "~/.config/"), self.prog + ".yml"
-            )
+            ),
+            os.path.join(USER_BASE, "etc", self.prog + ".yml"),
+            os.path.join("/usr/local/etc", self.prog + ".yml"),
+            os.path.join("/etc", self.prog + ".yml"),
         ]
 
 
@@ -315,10 +339,26 @@ class LoggingAction(argparse.Action):
 class UndertimeArgumentParser(ConfigArgumentParser):
     def __init__(self, *args, **kwargs):
         """override constructor to setup our arguments and config files"""
+        kwargs["add_help"] = False
         super().__init__(
             description="pick a meeting time", epilog=__doc__, *args, **kwargs
         )
+        # do not forget to change the manpage and shell completion in
+        # extra/ when you change anything below
         group = self.add_argument_group("time and date options")
+        group.add_argument(
+            "-d",
+            "--date",
+            default=None,
+            help=argparse.SUPPRESS,
+        )
+        group.add_argument(
+            "datelist",
+            default=None,
+            nargs="*",
+            metavar="WHEN",
+            help='target date for the meeting, for example "in two weeks" [default: now]',
+        )
         group.add_argument(
             "-t",
             "--timezones",
@@ -342,19 +382,6 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             metavar="HOUR",
             help="end of working day, in hours [default: %(default)s]",
         )
-        group.add_argument(
-            "-d",
-            "--date",
-            default=None,
-            help=argparse.SUPPRESS,
-        )
-        group.add_argument(
-            "datelist",
-            default=None,
-            nargs="*",
-            metavar="WHEN",
-            help='target date for the meeting, for example "in two weeks" [default: now]',
-        )
         group = self.add_argument_group("display options")
         group.add_argument(
             "--no-colors",
@@ -384,7 +411,7 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             "--no-overlap",
             "--overlap",
             action=NegateAction,
-            dest="overlap_show",
+            dest="overlap",
             help="show zones overlap [default: %(default)s]",
         )
         group.add_argument(
@@ -419,15 +446,22 @@ class UndertimeArgumentParser(ConfigArgumentParser):
         group.add_argument(
             "--no-table",
             "--table",
-            dest="show_table",
+            dest="table",
             action=NegateAction,
             help="hide/show the actual table [default: %(default)s]",
         )
+        # START HACK
+        #
+        # ConfigAction doesn't support subparsers and argument groups
+        # correctly. so we add this argument directly, which will end
+        # up in the first, default "optional arguments" group. Then,
+        # below, we hack at that group to coerce it in what we want.
         self.add_argument(
             "--config",
             action=YamlConfigAction,
             default=self.default_config(),
-            help="configuration file [default: %(default)s]",
+            help="configuration file [default: first of %s]"
+            % " ".join(self.default_config()),
         )
         self.add_argument(
             "-v",
@@ -442,6 +476,18 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             const="DEBUG",
             help="enable debugging messages",
         )
+        # this takes the default "optional arguments" group and throws
+        # it at the end of the list
+        #
+        # HACK: index 1 is a magic number, because self _optionals is
+        # defined right after self._positionals
+        default_group = self._action_groups.pop(1)
+        # change the title to something more consistent with the other groups
+        default_group.title = "other options"
+        # add it back at the end of the list
+        self._action_groups.append(default_group)
+        # END HACK
+
         group = self.add_argument_group("commands")
         group.add_argument(
             "-l",
@@ -458,6 +504,14 @@ class UndertimeArgumentParser(ConfigArgumentParser):
             action=ImportlibVersionAction,
             help="print version number and exit",
         )
+        group.add_argument(
+            "-h",
+            "--help",
+            action="help",
+            help="show this help message and exit",
+        )
+        # do not forget to change the manpage and shell completion in
+        # extra/ when you change anything above
 
     def parse_args(self, args=None, namespace=None):
         """override argument parser to properly interpret dates
@@ -589,26 +643,137 @@ fmt_time = fmt_time_colored
 
 
 def parse_date(date, local_zone):
+    """date parsing stub
+
+    This function delegates the parsing to an underlying module. Each
+    module is wrapped around by a stub function which hides all the
+    business logic behind that module's particular date parser.
+
+    This is split out in stubs this way so that we don't pay the
+    upfront "import" cost for all those modules if only one ends up
+    being used.
+
+    The parsers should therefore be ordered by load time cost.
+
+    Each stub is expected to handle exceptions from its own module,
+    and return False if it fails to import the module, or None if it
+    fails to parse the date.
+
+    A correctly parsed date should be returned as a datetime object.
+
+    When a new parser is added here, make sure to also report its
+    version in the --selftest output, currently in sysinfo().
+
+    Note that all date parsers fail in some way, each function details
+    the known failures.
+
+    """
+    now = None
     if date is None:
-        now = datetime.datetime.now(local_zone)
-    elif dateparser:
-        logging.debug("parsing date with dateparser module")
-        now = dateparser.parse(
-            date,
-            settings={"TIMEZONE": str(local_zone), "RETURN_AS_TIMEZONE_AWARE": True},
-        )
-    elif parsedatetime:  # pragma: no cover
-        logging.debug("parsing date with parsedatetime module")
-        cal = parsedatetime.Calendar()
-        now, parse_status = cal.parseDT(datetimeString=date, tzinfo=local_zone)
-        if not parse_status:
-            now = None
-    if now is None:
-        logging.warning("date provided cannot be parsed: %s", date)
-        now = datetime.datetime.now(local_zone)
+        return datetime.datetime.now(local_zone)
+    logging.debug("trying to parse date with dateparser module")
+    now = parse_date_dateparser(date, local_zone)
+    if now:
+        return now
+    logging.debug("trying to parse date with parsedatetime module")
+    now = parse_date_parsedatetime(date, local_zone)
+    if now:
+        return now
+    logging.debug("trying to parse date with arrow module")
+    now = parse_date_arrow(date)
+    if not now:
+        logging.error("date provided cannot be parsed: %s", date)
     return now
 
 
+def parse_date_dateparser(date, local_zone):
+    """parse date with the dateparser module
+
+    This can't parse things like "Next tuesday":
+
+    https://github.com/scrapinghub/dateparser/issues/573
+
+    It's also very slow, 300ms just on import:
+
+    https://github.com/scrapinghub/dateparser/issues/1051
+    """
+    try:
+        import dateparser
+    except ImportError:
+        return False
+    return dateparser.parse(
+        date,
+        settings={"TIMEZONE": str(local_zone), "RETURN_AS_TIMEZONE_AWARE": True},
+    )
+
+
+def parse_date_parsedatetime(date, local_zone):
+    """parse the given date with the parsedatetime module
+
+    This is fast, but doesn't parse timezones provided by the user:
+
+    https://github.com/bear/parsedatetime/issues/281
+
+    It's also a little bit *too* tolerate on date formats: if you pass
+    garbage alongside something that even looks like a valid date, it
+    will happily return you whatever it thinks is the date you
+    want. For example this works:
+
+    str(parsedatetime.Calendar().parseDT("garbled2022")[0]) == '2022-03-24 14:15:32'
+
+    ... and should probably just fail instead.
+    """
+    try:
+        import parsedatetime
+    except ImportError:
+        return False
+    cal = parsedatetime.Calendar()
+    now, parse_status = cal.parseDT(datetimeString=date, tzinfo=local_zone)
+    if not parse_status:
+        return None
+    else:
+        return now
+
+
+def parse_date_arrow(date):
+    """parse the given date with the arrrow module
+
+    This should be very rarely used, as dateparser can actually parse
+    most if not all the dates arrow can.
+
+    It does not support things like "next tuesday":
+
+    https://github.com/arrow-py/arrow/issues/1100
+
+    It also can't parse more "regular" date timestamps
+    (e.g. "2022-03-02"). The `get()` function can do that, but then
+    you need to specify a format, it can't guess.
+
+    For now it's just an ultimate fallback that could be useful if
+    someone is running without having installed the dateparser
+    dependency *and* happens to have arrow installed.
+
+    .. TODO:: arrow can handle a lot more things like relative date,
+    pytz and so on, so we could depend *only* on this instead of this
+    plethora of third-party modules we have right now...
+    """
+    try:
+        import arrow
+    except ImportError:
+        return False
+
+    if getattr(arrow, "dehumanize", False):  # 1.1.0 and later
+        logging.debug("parsing date with arrow module")
+        try:
+            return arrow.utcnow().dehumanize(date).datetime
+        except ValueError as e:
+            logging.debug("arrow failed to parsed date: %s", e)
+            return None
+    else:
+        logging.debug("arrow is to old to support the dehumanize method")
+        return False
+
+
 def flush_logging_handlers():
     """empty all buffered memory handler and yield their messages
 
@@ -652,8 +817,8 @@ def main(args):
     ║ 03:00  │    04:00     │ 08:00  │         0 ║
     ║ 04:00  │    05:00     │ 09:00_ │         1 ║
     ...
-    Local time requested: 2020-03-08 16:00:00+00:00
-    Equivalent to: 11:00 EST, 12:00 US/Eastern, 16:00 UTC
+    local time requested: 2020-03-08 16:00:00+00:00
+    equivalent to: 11:00 EST, 12:00 US/Eastern, 16:00 UTC
     >>> date = ['--date', '2020-11-01 12:00:00-04:00']
     >>> args = UndertimeArgumentParser().parse_args(argv + date + zones)
     >>> main(args)
@@ -671,8 +836,8 @@ def main(args):
     ║ 03:00  │    03:00     │ 08:00  │         0 ║
     ║ 04:00  │    04:00     │ 09:00_ │         1 ║
     ...
-    Local time requested: 2020-11-01 16:00:00+00:00
-    Equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
+    local time requested: 2020-11-01 16:00:00+00:00
+    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
     >>> extra = [ '--no-overlap', '--truncate' ]
     >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
     >>> main(args)
@@ -689,6 +854,18 @@ def main(args):
     ╠════════╪═══════════╪════════╪═════╣
     ║ 19:00  │   20:00   │ 00:00  │   0 ║
     ...
+    >>> extra = [ '--no-table' ]
+    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
+    >>> main(args)
+    UTC time requested: 2020-11-01 16:00:00+00:00
+    local time requested: 2020-11-01 16:00:00+00:00
+    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC
+    >>> zones = ['--timezone', 'EST', 'US/Eastern', 'UTC', 'Pacific/Auckland']
+    >>> args = UndertimeArgumentParser().parse_args(argv + date + zones + extra)
+    >>> main(args)
+    UTC time requested: 2020-11-01 16:00:00+00:00
+    local time requested: 2020-11-01 16:00:00+00:00
+    equivalent to: 11:00 EST, 11:00 US/Eastern, 16:00 UTC, 2020-11-02 05:00 Pacific/Auckland
     >>> logging.getLogger('').setLevel('WARNING')
     >>> argv = ['--config', '/dev/null', '--no-default']
     >>> args = UndertimeArgumentParser().parse_args(argv + date + [ '--timezones', 'CEST' ])
@@ -703,6 +880,12 @@ def main(args):
     Traceback (most recent call last):
         ...
     SystemExit: 1
+    >>> argv = ['--config', '/dev/null', '--no-default']
+    >>> args = UndertimeArgumentParser().parse_args(argv + ["NOTADATE"])
+    >>> main(args)
+    Traceback (most recent call last):
+        ...
+    SystemExit: 1
     >>> msgs = flush_logging_handlers()
     >>> assert 'unknown zone, skipping: CEST' in msgs
     >>> assert 'No valid time zone found.' in msgs
@@ -729,7 +912,14 @@ def main(args):
     # find the local timezone
     now_zone = get_localzone()
     # make an educated guess at what the user meant by passing that time zone to parse_date
-    then_local = parse_date(args.date, now_zone).replace(second=0, microsecond=0)
+    then_local = parse_date(args.date, now_zone)
+    if not then_local:
+        sys.exit(1)
+    # round off microseconds to cleanup display, in case we use "now"
+    # which *will* have microseconds. (other parsed times will most
+    # likely not have microseconds, and who cares about *those*
+    # anyways... that shouldn't matter, right? .... RIGHT?)
+    then_local = then_local.replace(microsecond=0)
     # convert that time to UTC again
     then_utc = then_local.astimezone(datetime.timezone.utc)
     # and guess what time zone that was *then*
@@ -746,14 +936,14 @@ def main(args):
         logging.error("No valid time zone found.")
         sys.exit(1)
 
-    if args.show_table:
+    if args.table:
         rows = compute_table(
             then_local,
             timezones,
             args.start,
             args.end,
             overlap_min=args.overlap_min,
-            overlap_show=args.overlap_show,
+            overlap_show=args.overlap,
             truncate=args.truncate,
         )
         # reproduce the terminaltables DoubleTable output in tabulate:
@@ -775,12 +965,23 @@ def main(args):
         print(table)
     times = []
     for zone in timezones:
-        times.append(
-            "{0:%H:%M} {1}".format(then_local.astimezone(tz=zone).timetz(), zone)
-        )
+        other_zone_day = "{0:%Y-%m-%d}".format(then_local.astimezone(tz=zone))
+        local_zone_day = "{0:%Y-%m-%d}".format(then_local)
+        if local_zone_day != other_zone_day:
+            logging.debug(
+                "different day: local (%s) is %s other (%s) is %s",
+                then_zone,
+                local_zone_day,
+                zone,
+                other_zone_day,
+            )
+            ts = "{0:%Y-%m-%d %H:%M} {1}".format(then_local.astimezone(tz=zone), zone)
+        else:
+            ts = "{0:%H:%M} {1}".format(then_local.astimezone(tz=zone).timetz(), zone)
+        times.append(ts)
     print("UTC time requested: {}".format(then_utc))
-    print("Local time requested: {}".format(then_local))
-    print("Equivalent to: " + ", ".join(times))
+    print("local time requested: {}".format(then_local))
+    print("equivalent to: " + ", ".join(times))
 
 
 def get_localzone(now_utc=None):
@@ -975,11 +1176,84 @@ def compute_table(
     return rows
 
 
+def sysinfo(callback=logging.info):  # pragma: nocover
+    """dump a lot of system information to help with support"""
+    callback("file: %s", __file__)
+    callback("command: %r", sys.argv)
+    callback("version: %s", ImportlibVersionAction._version())
+    callback(
+        "%s: %s (%s %s)",
+        platform.python_implementation(),
+        platform.python_version(),
+        platform.python_compiler(),
+        " ".join(platform.python_build()),
+    )
+    callback("kernel: %s", " ".join(platform.uname()))
+    if "linux" in platform.system().lower():
+        try:
+            distribution = platform.freedesktop_os_release()["PRETTY_NAME"]
+        except AttributeError:
+            # Python < 3.10 poor man's freedesktop_os_release(). we
+            # don't vendor the thing in here because it's too big
+            try:
+                with open("/etc/os-release") as fp:
+                    for line in fp.readlines():
+                        if line.startswith("PRETTY_NAME"):
+                            distribution = line.split("=")[1].strip().strip('"')
+                            break
+                    else:
+                        distribution = ""
+            except Exception as e:
+                distribution = "failed to find distribution: %s" % e
+    else:
+        distribution = ""
+    callback("operating system: %s (%s)", platform.system(), distribution)
+    # we import from in the global import, reimport to access the version
+    import dateutil
+
+    for mod in (
+        dateutil,
+        pytz,
+        tabulate,
+        termcolor,
+        yaml,
+    ):
+        version = getattr(mod, "__version__", None) or getattr(mod, "VERSION", None)
+        # termcolor likes their versions as tuples of integers, argh.
+        if type(version) is tuple:
+            version = ".".join([str(x) for x in version])
+        callback(
+            "module %s-%s from '%s'",
+            mod.__name__,
+            version,
+            mod.__file__,
+        )
+
+    for dateparser in ("dateparser", "parsedatetime", "arrow"):
+        try:
+            mod = importlib.import_module(dateparser)
+        except ModuleNotFoundError:
+            callback("module %s not found", dateparser)
+            continue
+        callback(
+            "module %s-%s from '%s'",
+            mod.__name__,
+            getattr(mod, "__version__", None),
+            mod.__file__,
+        )
+
+    callback("timezone database version: %s", pytz.OLSON_VERSION)
+
 if __name__ == "__main__":  # pragma: nocover
     logging.basicConfig(format="%(levelname)s: %(message)s", level="WARNING")
     parser = UndertimeArgumentParser()
     args = parser.parse_args()
     if args.selftest:
+
+        def print_as_log(format, *args):
+            print(format % args)
+
+        sysinfo(callback=print_as_log)
         # reset loggers
         logging.getLogger().handlers = []
         # log to memory so that doctest can check output and we do not
@@ -997,6 +1271,10 @@ if __name__ == "__main__":  # pragma: nocover
         import doctest
 
         result = doctest.testmod(optionflags=doctest.ELLIPSIS)
+        if result.failed:
+            print("test suite failed")
+        else:
+            print("test suite succeeded")
         sys.exit(result.failed)
 
     main(args)
